@@ -351,24 +351,6 @@ bool mysql_derived_merge(THD *thd, LEX *lex, TABLE_LIST *derived)
     DBUG_RETURN(FALSE);
   }
 
-  if (dt_select->uncacheable & UNCACHEABLE_RAND)
-  {
-    /* There is random function => fall back to materialization. */
-    cause= "Random function in the select";
-    if (unlikely(thd->trace_started()))
-    {
-      OPT_TRACE_VIEWS_TRANSFORM(thd, trace_wrapper, trace_derived,
-                          derived->is_derived() ? "derived" : "view",
-                          derived->alias.str ? derived->alias.str : "<NULL>",
-                          derived->get_unit()->first_select()->select_number,
-                          "materialized");
-      trace_derived.add("cause", cause);
-    }
-    derived->change_refs_to_fields();
-    derived->set_materialized_derived();
-    DBUG_RETURN(FALSE);
-  }
-
   if (derived->dt_handler)
   {
     derived->change_refs_to_fields();
@@ -805,6 +787,9 @@ bool mysql_derived_prepare(THD *thd, LEX *lex, TABLE_LIST *derived)
         cursor->outer_join|= JOIN_TYPE_OUTER;
     }
   }
+  // Prevent it for possible ORDER BY clause
+  if (unit->fake_select_lex)
+    unit->fake_select_lex->context.outer_context= 0;
 
   if (unlikely(thd->trace_started()))
   {
@@ -1157,7 +1142,14 @@ bool TABLE_LIST::fill_recursive(THD *thd)
   while (!rc && !with->all_are_stabilized())
   {
     if (with->level > thd->variables.max_recursive_iterations)
+    {
+      push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
+                          ER_QUERY_RESULT_INCOMPLETE,
+                          ER_THD(thd, ER_QUERY_RESULT_INCOMPLETE),
+                          "max_recursive_iterations =",
+                          (ulonglong)thd->variables.max_recursive_iterations);
       break;
+    }
     with->prepare_for_next_iteration();
     rc= unit->exec_recursive();
   }
