@@ -45,7 +45,7 @@ public:
     , m_server_status(thd->server_status)
   {
     m_thd->variables.option_bits&= ~OPTION_BEGIN;
-    m_thd->server_status&= ~SERVER_STATUS_IN_TRANS;
+    m_thd->server_status&= ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
     m_thd->wsrep_cs().enter_toi_mode(ws_meta);
   }
   ~Wsrep_non_trans_mode()
@@ -357,8 +357,6 @@ int Wsrep_high_priority_service::rollback(const wsrep::ws_handle& ws_handle,
   m_thd->wsrep_cs().prepare_for_ordering(ws_handle, ws_meta, false);
   int ret= (trans_rollback_stmt(m_thd) || trans_rollback(m_thd));
   m_thd->release_transactional_locks();
-  mysql_ull_cleanup(m_thd);
-  m_thd->mdl_context.release_explicit_locks();
 
   free_root(m_thd->mem_root, MYF(MY_KEEP_PREALLOC));
 
@@ -472,20 +470,24 @@ int Wsrep_high_priority_service::log_dummy_write_set(const wsrep::ws_handle& ws_
     if (!WSREP_EMULATE_BINLOG(m_thd))
     {
       wsrep_register_for_group_commit(m_thd);
-      ret = ret || cs.provider().commit_order_leave(ws_handle, ws_meta, err);
+      /* wait_for_prior_commit() ensures that all preceding transactions
+         have been committed and seqno has been synced into
+         storage engine. We don't release commit order here yet to
+         avoid following transactions to sync seqno before
+         wsrep_set_SE_checkpoint() below returns. This effectively pauses
+         group commit for the checkpoint operation, but is the only way to
+         ensure proper ordering. */
       m_thd->wait_for_prior_commit();
     }
 
+    WSREP_DEBUG("checkpointing dummy write set %lld", ws_meta.seqno().get());
     wsrep_set_SE_checkpoint(ws_meta.gtid());
 
     if (!WSREP_EMULATE_BINLOG(m_thd))
     {
       wsrep_unregister_from_group_commit(m_thd);
     }
-    else
-    {
-      ret= ret || cs.provider().commit_order_leave(ws_handle, ws_meta, err);
-    }
+    ret= ret || cs.provider().commit_order_leave(ws_handle, ws_meta, err);
     cs.after_applying();
   }
   DBUG_RETURN(ret);
