@@ -445,12 +445,6 @@ public:
     IO_CACHE *file,
     uint32 arg_length
   );
-  bool append_with_prefill(
-    const char *s,
-    uint32 arg_length,
-    uint32 full_length,
-    char fill_char
-  );
   int strstr(
     const String &search,
     uint32 offset = 0
@@ -813,8 +807,8 @@ class spider_db_util
 {
 public:
   uint dbton_id;
-  spider_db_util() {}
-  virtual ~spider_db_util() {}
+  spider_db_util() = default;
+  virtual ~spider_db_util() = default;
   virtual int append_name(
     spider_string *str,
     const char *name,
@@ -949,7 +943,7 @@ public:
   uint dbton_id;
   SPIDER_DB_ROW *next_pos;
   spider_db_row(uint in_dbton_id) : dbton_id(in_dbton_id), next_pos(NULL) {}
-  virtual ~spider_db_row() {}
+  virtual ~spider_db_row() = default;
   virtual int store_to_field(
     Field *field,
     CHARSET_INFO *access_charset
@@ -981,8 +975,8 @@ public:
 class spider_db_result_buffer
 {
 public:
-  spider_db_result_buffer() {}
-  virtual ~spider_db_result_buffer() {}
+  spider_db_result_buffer() = default;
+  virtual ~spider_db_result_buffer() = default;
   virtual void clear() = 0;
   virtual bool check_size(
     longlong size
@@ -996,7 +990,7 @@ protected:
 public:
   uint dbton_id;
   spider_db_result(SPIDER_DB_CONN *in_db_conn);
-  virtual ~spider_db_result() {}
+  virtual ~spider_db_result() = default;
   virtual bool has_result() = 0;
   virtual void free_result() = 0;
   virtual SPIDER_DB_ROW *current_row() = 0;
@@ -1062,7 +1056,7 @@ public:
   spider_db_conn(
     SPIDER_CONN *in_conn
   );
-  virtual ~spider_db_conn() {}
+  virtual ~spider_db_conn() = default;
   virtual int init() = 0;
   virtual bool is_connected() = 0;
   virtual void bg_connect() = 0;
@@ -1281,7 +1275,7 @@ public:
     st_spider_share *share,
     uint dbton_id
   ) : dbton_id(dbton_id), spider_share(share) {}
-  virtual ~spider_db_share() {}
+  virtual ~spider_db_share() = default;
   virtual int init() = 0;
   virtual uint get_column_name_length(
     uint field_index
@@ -1328,7 +1322,7 @@ public:
   spider_db_handler(ha_spider *spider, spider_db_share *db_share) :
     dbton_id(db_share->dbton_id), spider(spider), db_share(db_share),
     first_link_idx(-1) {}
-  virtual ~spider_db_handler() {}
+  virtual ~spider_db_handler() = default;
   virtual int init() = 0;
   virtual int append_index_hint(
     spider_string *str,
@@ -1633,9 +1627,6 @@ public:
     SPIDER_HS_STRING_REF &info
   ) = 0;
 #endif
-  virtual bool need_lock_before_set_sql_for_exec(
-    ulong sql_type
-  ) = 0;
 #ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int set_sql_for_exec(
     ulong sql_type,
@@ -1815,7 +1806,7 @@ public:
   spider_db_share *db_share;
   spider_db_copy_table(spider_db_share *db_share) :
     dbton_id(db_share->dbton_id), db_share(db_share) {}
-  virtual ~spider_db_copy_table() {}
+  virtual ~spider_db_copy_table() = default;
   virtual int init() = 0;
   virtual void set_sql_charset(
     CHARSET_INFO *cs
