@@ -22,22 +22,33 @@ public:
     spider_string *str,
     const char *name,
     uint name_length
-  );
+  ) override;
   int append_name_with_charset(
     spider_string *str,
     const char *name,
     uint name_length,
     CHARSET_INFO *name_charset
-  );
+  ) override;
+  int append_escaped_name(
+    spider_string *str,
+    const char *name,
+    uint name_length
+  ) override;
+  int append_escaped_name_with_charset(
+    spider_string *str,
+    const char *name,
+    uint name_length,
+    CHARSET_INFO *name_charset
+  ) override;
   bool is_name_quote(
     const char head_code
-  );
+  ) override;
   int append_escaped_name_quote(
     spider_string *str
-  );
+  ) override;
   int append_column_value(ha_spider *spider, spider_string *str, Field *field,
                           const uchar *new_ptr, bool is_like,
-                          CHARSET_INFO *access_charset);
+                          CHARSET_INFO *access_charset) override;
   int append_from_with_alias(
     spider_string *str,
     const char **table_names,
@@ -51,29 +62,41 @@ public:
   int append_trx_isolation(
     spider_string *str,
     int trx_isolation
-  );
+  ) override;
   int append_autocommit(
     spider_string *str,
     bool autocommit
-  );
+  ) override;
   int append_sql_log_off(
     spider_string *str,
     bool sql_log_off
+  ) override;
+  int append_wait_timeout(
+    spider_string *str,
+    int wait_timeout
+  ) override;
+  virtual int append_sql_mode_internal(
+    spider_string *str,
+    sql_mode_t sql_mode
   );
+  int append_sql_mode(
+    spider_string *str,
+    sql_mode_t sql_mode
+  ) override;
   int append_time_zone(
     spider_string *str,
     Time_zone *time_zone
-  );
+  ) override;
   int append_start_transaction(
     spider_string *str
-  );
+  ) override;
   int append_xa_start(
     spider_string *str,
     XID *xid
-  );
+  ) override;
   int append_lock_table_head(
     spider_string *str
-  );
+  ) override;
   int append_lock_table_body(
     spider_string *str,
     const char *db_name,
@@ -83,13 +106,13 @@ public:
     uint table_name_length,
     CHARSET_INFO *table_name_charset,
     int lock_type
-  );
+  ) override;
   int append_lock_table_tail(
     spider_string *str
-  );
+  ) override;
   int append_unlock_table(
     spider_string *str
-  );
+  ) override;
   int open_item_func(
     Item_func *item_func,
     ha_spider *spider,
@@ -98,7 +121,26 @@ public:
     uint alias_length,
     bool use_fields,
     spider_fields *fields
+  ) override;
+protected:
+  int check_item_func(
+    Item_func *item_func,
+    ha_spider *spider,
+    const char *alias,
+    uint alias_length,
+    bool use_fields,
+    spider_fields *fields
   );
+  int print_item_func(
+    Item_func *item_func,
+    ha_spider *spider,
+    spider_string *str,
+    const char *alias,
+    uint alias_length,
+    bool use_fields,
+    spider_fields *fields
+  );
+public:
 #ifdef HANDLER_HAS_DIRECT_AGGREGATE
   int open_item_sum_func(
     Item_sum *item_sum,
@@ -108,12 +150,12 @@ public:
     uint alias_length,
     bool use_fields,
     spider_fields *fields
-  );
+  ) override;
 #endif
   int append_escaped_util(
     spider_string *to,
     String *from
-  );
+  ) override;
 #ifdef SPIDER_HAS_GROUP_BY_HANDLER
   int append_table(
     ha_spider *spider,
@@ -155,18 +197,18 @@ public:
     spider_string *str,
     TABLE_LIST *table_list,
     uint table_count
-  );
+  ) override;
   int reappend_tables(
     spider_fields *fields,
     SPIDER_LINK_IDX_CHAIN *link_idx_chain,
     spider_string *str
-  );
+  ) override;
   int append_where(
     spider_string *str
-  );
+  ) override;
   int append_having(
     spider_string *str
-  );
+  ) override;
 #endif
 };
 
@@ -175,6 +217,14 @@ class spider_db_mysql_util: public spider_db_mbase_util
 public:
   spider_db_mysql_util();
   ~spider_db_mysql_util();
+  int append_column_value(
+    ha_spider *spider,
+    spider_string *str,
+    Field *field,
+    const uchar *new_ptr,
+    bool is_like,
+    CHARSET_INFO *access_charset
+  ) override;
 };
 
 class spider_db_mariadb_util: public spider_db_mbase_util
@@ -182,6 +232,18 @@ class spider_db_mariadb_util: public spider_db_mbase_util
 public:
   spider_db_mariadb_util();
   ~spider_db_mariadb_util();
+  int append_sql_mode_internal(
+    spider_string *str,
+    sql_mode_t sql_mode
+  ) override;
+  int append_column_value(
+    ha_spider *spider,
+    spider_string *str,
+    Field *field,
+    const uchar *new_ptr,
+    bool is_like,
+    CHARSET_INFO *access_charset
+  ) override;
 };
 
 class spider_db_mbase_row: public spider_db_row
@@ -263,20 +325,22 @@ public:
   );
   int fetch_table_status(
     int mode,
-    ha_rows &records,
-    ulong &mean_rec_length,
-    ulonglong &data_file_length,
-    ulonglong &max_data_file_length,
-    ulonglong &index_file_length,
-    ulonglong &auto_increment_value,
-    time_t &create_time,
-    time_t &update_time,
-    time_t &check_time
+    ha_statistics &stat
+  );
+  int fetch_simple_action(
+    uint simple_action,
+    uint position,
+    void *param
   );
   int fetch_table_records(
     int mode,
     ha_rows &records
   );
+#ifdef HA_HAS_CHECKSUM_EXTENDED
+  int fetch_table_checksum(
+    ha_spider *spider
+  );
+#endif
   int fetch_table_cardinality(
     int mode,
     TABLE *table,
@@ -392,9 +456,7 @@ public:
   bool is_xa_nota_error(
     int error_num
   );
-  void print_warnings(
-    struct tm *l_time
-  );
+  void fetch_and_print_warnings(struct tm *l_time);
   spider_db_result *store_result(
     spider_db_result_buffer **spider_res_buf,
     st_spider_db_request_key *request_key,
@@ -406,6 +468,11 @@ public:
   );
   int next_result();
   uint affected_rows();
+  uint matched_rows();
+  bool inserted_info(
+    spider_db_handler *handler,
+    ha_copy_info *copy_info
+  );
   ulonglong last_insert_id();
   int set_character_set(
     const char *csname
@@ -462,6 +529,16 @@ public:
     bool sql_log_off,
     int *need_mon
   );
+  bool set_wait_timeout_in_bulk_sql();
+  int set_wait_timeout(
+    int wait_timeout,
+    int *need_mon
+  );
+  bool set_sql_mode_in_bulk_sql();
+  int set_sql_mode(
+    sql_mode_t sql_mode,
+    int *need_mon
+  );
   bool set_time_zone_in_bulk_sql();
   int set_time_zone(
     Time_zone *time_zone,
@@ -656,6 +733,9 @@ public:
     spider_string *str
   );
 #endif
+#ifdef HA_HAS_CHECKSUM_EXTENDED
+  bool checksum_support();
+#endif
 protected:
   int create_table_names_str();
   void free_table_names_str();
@@ -741,6 +821,7 @@ public:
   spider_mbase_share      *mysql_share;
   SPIDER_LINK_FOR_HASH    *link_for_hash;
   uchar                   *minimum_select_bitmap;
+  uchar                   direct_insert_kind;
   spider_mbase_handler(
     ha_spider *spider,
     spider_mbase_share *share,
@@ -1395,9 +1476,6 @@ public:
     SPIDER_HS_STRING_REF &info
   );
 #endif
-  bool need_lock_before_set_sql_for_exec(
-    ulong sql_type
-  );
 #ifdef SPIDER_HAS_GROUP_BY_HANDLER
   int set_sql_for_exec(
     ulong sql_type,
@@ -1435,9 +1513,18 @@ public:
     int link_idx,
     int crd_mode
   );
+  int simple_action(
+    uint simple_action,
+    int link_idx
+  );
   int show_records(
     int link_idx
   );
+#ifdef HA_HAS_CHECKSUM_EXTENDED
+  int checksum_table(
+    int link_idx
+  );
+#endif
   int show_last_insert_id(
     int link_idx,
     ulonglong &last_insert_id
